# from asyncio.log import logger
import logging

from fastapi import HTTPException
import requests
from starlette.status import HTTP_401_UNAUTHORIZED
from starlette_context import context
import json
import conf
import correlation

logger = logging.getLogger(__name__)


class EntitlementService:
    def request_groups(self, headers, timeout=10):
        # Just return canned result of Entitlement Groups API
        if conf.MOCK_ENTITLEMENT:
            return conf.MOCK_ENTITLEMENT_RESULT

        logging.setLogRecordFactory(
            correlation.set_correlation_id(context["correlation_id"])
        )
        logger = logging.getLogger(__name__)

        logger.debug(f"{conf.ENTITLEMENTS_GROUPS_API} headers: {headers}")
        try:
            response = requests.get(
                conf.ENTITLEMENTS_GROUPS_API, headers=headers, timeout=timeout
            )
        except requests.exceptions.RequestException as e:
            logger.warning(f"URL:{conf.ENTITLEMENTS_GROUPS_API} returned: {e}")
            raise HTTPException(
                status_code=HTTP_401_UNAUTHORIZED,
                detail=f"Authentication was unsuccessful. Unexpected response from entitlement {e}",
                headers={"WWW-Authenticate": "Bearer"},
            )

        if not response.ok:
            logger.warning(
                f"URL:{conf.ENTITLEMENTS_GROUPS_API} returned: {response.text} {response.status_code}"
            )
            message = ""
            if "message" in response.text:
                message = json.loads(response.text)["message"]
            raise HTTPException(
                status_code=HTTP_401_UNAUTHORIZED,
                detail=f"Authentication was unsuccessful. {message} {response.status_code}",
                headers={"WWW-Authenticate": "Bearer"},
            )

        jdata = response.json()
        if "groups" in jdata:
            return jdata

        # this should never happen
        raise HTTPException(
            status_code=HTTP_401_UNAUTHORIZED,
            detail="Authentication was unsuccessful. Groups undefined",
            headers={"WWW-Authenticate": "Bearer"},
        )
