# test_integration_015_legal.py
import pytest

# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
from auth import auth
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

TEST_DATA_DIR = Path(__file__).resolve().parent / "data"

client = TestClient(app)


@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"


def test_create_legal_tag(token, data_partition):
    """Test Create a legal tag"""

    if conf.MOCK_ENTITLEMENT:
        pytest.skip("MOCK ENTITLEMENT ENABLED")

    url = conf.LEGAL_BASE_URL + "/api/legal/v1/legaltags?valid=true"

    legal_tag = {
        "name": "osdu-usa-dataset-osduonaws-testing",
        "properties": {
            "countryOfOrigin": ["US"],
            "contractId": "A1234",
            "expirationDate": 2222222222222,
            "originator": "Default",
            "dataType": "Public Domain Data",
            "securityClassification": "Public",
            "personalData": "No Personal Data",
            "exportClassification": "EAR99",
        },
        "description": "A default legal tag",
    }

    headers = {"Authorization": "Bearer " + token, "data-partition-id": data_partition}

    # create default legal tag to avoid empty legal tag response
    r = requests.post(url, headers=headers, json=legal_tag)
    assert (
        r.status_code == 201 or r.status_code == 409
    ), f"Expect can add tag {r.text} {r.status_code}"
