# testlib.py
import os
import requests
import re
from string import Template
import pytest
import conf

def put_policies_test_data(
    client,
    token,
    data_partition,
    data_partition_header=None,
    service_url=None,
    domain="example.com"):

    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, 'templates')
    for filename in sorted(os.listdir(template_datadir)):
        if filename.endswith(".rego"):
            put_policy_test_data(client=client,
                             token=token,
                             data_partition=data_partition,
                             filename=filename,
                             data_partition_header=data_partition_header,
                             service_url=service_url,
                             domain=domain)

def put_policy_test_data(
    client,
    token,
    data_partition,
    filename,
    data_partition_header=None,
    service_url=None,
    domain="example.com",
    validate=False):
    """
    This supporting function takes
    client = FastAPI test client or requests
    token = token to use for testing put in header of request
    data_partition_header = data partition put in header of request
    data_partition = data_partition to put in path parameter and header request if data_partition_header
    domain: domain name
    """
    if not data_partition_header:
        data_partition_header = data_partition

    num_tests = 0
    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, 'templates')

    filetype='.rego'
    file_path = os.path.join(template_datadir, filename)
    # checking if it is a file
    if os.path.isfile(file_path):
        #file_path.endswith(filetype) or file_path.endswith(filetype+".template"):
        if file_path.endswith(filetype):
            policy_id = os.path.basename(file_path)
            policy_id_short = policy_id.removesuffix(filetype)
            print(policy_id)
            #policy_id = policy_id.removesuffix(filetype)
            #f = open(file_path, "rb")
            #files = {'file': open(file_path, 'rb')}
            #encode_str = base64.b64encode (bytes(f.read(), "utf-8"))
            print(f"file: {file_path}, policy_id: {policy_id}")
            f = open(file_path, 'r')
            data = f.read()
            f.close()
            #data = strip_comments(data)
            template_data = Template(data)
            data = template_data.substitute(
                {
                    'data_partition': data_partition,
                    'DATA_PARTITION': data_partition,
                    'data_partition_id': data_partition,
                    'name': policy_id_short,
                    'domain': domain,
                    'DOMAIN': domain
                })

            if f"package osdu.partition[\"{data_partition}\"].{policy_id_short}" not in data:
                print(f"expected matching package name '{policy_id_short}' for {file_path} skipping")
                pytest.xfail(f"expected matching package name '{policy_id_short}' for {file_path}")

            assert f"package osdu.partition[\"{data_partition}\"].{policy_id_short}" in data, f"expected matching package name '{policy_id_short}' for {file_path}"
            bdata = data.encode('utf-8')
            files = {'file': (policy_id, bdata)}

            if validate:
                url = conf.SERVICE_BASE_PATH + "/validate/" + policy_id
            else:
                url = conf.SERVICE_BASE_PATH + "/policies/osdu/partition/" + data_partition + "/" + policy_id

            if service_url:
                url = service_url + url
                r = requests.put(url,
                    files = files,
                    headers={'Authorization': 'Bearer ' + token,
                    'data-partition-id': data_partition_header})
            else:
                r = client.put(url,
                    files = files,
                    headers={'Authorization': 'Bearer ' + token,
                    'data-partition-id': data_partition_header})
            if validate:
                if r.status_code == 422 or r.status_code == 400:
                    # Invalid rego. Let's just stop now
                    pytest.fail(f"{os.environ['PYTEST_CURRENT_TEST']} Failed validation! {r.status_code} {r.text}")
                assert r.status_code == 200 or r.status_code == 202, f"Expected to get 200 or 202, got {r.status_code} from {url} {r.text}"
                assert " valid " in r.text, f"Expect 'valid' in message of result but found {r.text}"
            else:
                assert r.status_code == 200 or r.status_code == 202, f"Expected to get 200 or 202, got {r.status_code} from {url} {r.text}"
                assert "added to " in r.text or "updated in" in r.text or "already in" in r.text, f"Expect 'added to... or updated in...' message in result but found {r.text}"
                assert policy_id in r.text, f"Expect {policy_id} message in result {r.text}"
                result = r.json()
                #print(f"returned json: {json.dumps(result, indent=4)}")
                assert result["status"] is True, "Expected status true (success)"
            num_tests = num_tests + 1
        else:
            pytest.skip("Only .rego templates")
    #assert num_tests >= 7, "At least 7 polices were tested"

def strip_comments(code):
    # strip comments out of templates
    code = str(code)
    return re.sub(r'(?m)^ *#.*\n?', '', code)

def get_list_legal_tags(token, data_partition):
    url = conf.LEGAL_BASE_URL + "/api/legal/v1/legaltags?valid=true"
    r = requests.get(url,
        headers={'Authorization': 'Bearer ' + token,
        'data-partition-id': data_partition})
    #print(json.dumps(r.json(), indent=4))

    return r.json()["legalTags"]

def get_domain_name(token, data_partition):
    """
    return domainname without the data_partition hostname
    """
    url = conf.ENTITLEMENTS_GROUPS_API
    r = requests.get(url,
        headers={'Authorization': 'Bearer ' + token,
        'data-partition-id': data_partition})
    jdata = r.json()
    if "groups" in jdata:
        domain=jdata['groups'][0]["email"].split('@')[1]
        domain_ending = domain.removeprefix(data_partition + ".")
        return domain_ending

def get_rego_templates():
    p = os.path.dirname(os.path.abspath(__file__))
    template_datadir = os.path.join(p, 'templates')

    list_of_files = sorted(os.listdir(template_datadir))
    #rego_files = [x for x in list_of_files if x.endswith(".rego") or x.endswith(".rego.template")]
    rego_files = [x for x in list_of_files if x.endswith(".rego")]
    return rego_files
