import os
import tarfile
from azure.storage.blob import BlobServiceClient
from azure.storage.blob import ContentSettings
from Helper import Helper
from pip._internal.utils._log import getLogger

logger = getLogger(__name__)

class BootstrapBundles:

    def __init__(self):
        logger.info("Initializing AzureBlobFileUploader")
        # Initialize the connection to Azure storage account
        helper = Helper()
        self.blob_service_client = BlobServiceClient.from_connection_string(
            helper.get_storage_connection_string())

    def create_bundles(self, path, tar_name):
        with tarfile.open(tar_name, "w:gz") as tar_handle:
            for root, dirs, files in os.walk(path):
                for file in files:
                    if file == "legal.rego" or file == "entitlements.rego" or file == "dataauthz.rego" or file == ".manifest" or file == "search.rego":
                        tar_handle.add(os.path.join(root, file))

    def upload_bundles(self):
        # Create blob with same name as local file name
        blob_client = self.blob_service_client.get_blob_client(container=Helper.CONTAINER_NAME,
                                                               blob=Helper.BUNDLE_FILE_NAME)

        # Create blob on storage
        # Overwrite if it already exists!
        image_content_setting = ContentSettings(content_type='application/x-gtar')
        logger.info("Uploading file - %s",Helper.BUNDLE_FILE_NAME)
        with open(Helper.BUNDLE_FILE_NAME, "rb") as data:
            try:
                blob_client.upload_blob(data, overwrite=True, content_settings=image_content_setting)
            except Exception as e:
                logger.error("Upload to Bundle Server failed %s", e)
                raise RuntimeError(e)



# Initialize class and upload bundles
if __name__ == '__main__':
    BootstrapBundles.create_bundles(Helper.DEFAULT_POLICIES_PATH, Helper.BUNDLE_FILE_NAME)
    BootstrapBundles().upload_bundles()
